Unit Ggauge;

{
  TGradGauge v1.05 by Doodu Ohana  11/98
  Second version by SeBco soft (Sergey V. Belov) 10/2000

  This gauge is filled with solid vertical gradient bar,
  marking the % area already done. You can also add text.

  Properties:
   Gradient colors - start (on top) & end (on bottom) colors;
   Background color;
   Min, Max & Progress values;
   Caption;
   Hint;
   AutoText (new) - auto change text to template "progress / max";
   Mouse events;

   Methods:
    Inc (new) - increase progress by 1;
    Dec (new) - decrease progress by 1;

   Vertical gradient is better than horizontal in two ways:
   - It looks better - you can see the gradient also when the
     bar is not full.
   - It's faster (I hope).

   Solid bar is better then non-solid (cubes) because you see text
   better on it.

   This component is absolute FREEWARE. It's a result of my work on
   a project, so it fits my needs and not very customizable. Change
   it as you like and please send me the results. I hope it is bug
   free.

   I decided to release this component (and others) to the public as
   a contribute to the Delphi programmers community, since I have used
   many other's components in my project.


   Contact me: oohana@ort.org.il

   (C) SeBco soft,2000
   Some updates.
   belser@chat.ru
   }



Interface

Uses
  Windows, Messages, Sysutils,
  Classes, Graphics, Controls, Forms, Dialogs;

 Const
   Arc1 = 10;

 Type
   Tgradgauge = Class(Tgraphiccontrol)
   Private
   Fbmp: TBitmap;
   Fbgcolor, Fscolor, Fecolor: Tcolor;
   Fprogress, Fmax, Fmin : Cardinal;
   Fautocap: Boolean;
   FRepresentationKind: Boolean;
   Text: String;

   Protected
   Procedure Setcolor1 (Value: Tcolor);
   Procedure Setcolor2 (Value: Tcolor);
   Procedure Setbgcolor (Value: Tcolor);
   Procedure Setmin (Value: Cardinal);
   Procedure Setmax (Value: Cardinal);
   Procedure Setprogress (Value: Cardinal);
   Procedure Settext (Value: String);
   Procedure Setautocap (Value: Boolean);
   Procedure Gradfill(Clr1, Clr2: Tcolor; Thebitmap: TBitmap);
   Procedure SetRepresentation(Value: Boolean);
   Procedure Paint; Override;
   Public
   Constructor Create(Aowner: Tcomponent); Override;
   Destructor Destroy; Override;
   procedure inc;
   procedure dec;
  Published
    Property Backcolor:    Tcolor Read Fbgcolor Write Setbgcolor;
    Property Color1:       Tcolor Read Fscolor Write Setcolor1;
    Property Color2:       Tcolor Read Fecolor Write Setcolor2;
    Property Min:          Cardinal Read Fmin Write Setmin;
    Property Max:          Cardinal Read Fmax Write Setmax;
    Property Progress:     Cardinal Read Fprogress Write Setprogress;
    Property Caption:      String Read Text Write Settext;
    Property Autocaption:  Boolean Read Fautocap Write Setautocap;
    Property AutoSwitchTextPercents: Boolean Read FRepresentationKind Write SetRepresentation;

    Property Visible;
    Property Font;
    Property OnClick;
    Property OnDblClick;
    Property OnDragDrop;
    Property OnDragOver;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
    Property Hint;
    Property ShowHint;
  End;

Procedure Register;

Implementation
Var
  Percent, Rp: Cardinal;
  R: TRect;
  Refreshcap: Boolean;

Constructor Tgradgauge.Create(Aowner: Tcomponent);
Begin
  Inherited Create(Aowner);
  { default values }
  Width := 200;
  Height := 20;
  Fmin := 0;
  Fmax := 100;
  Fprogress := Fmin;
  FRepresentationKind := true;

  Fscolor := ClNavy;
  Fecolor := ClAqua;
  Fbgcolor := ClBlack;

  Fbmp := TBitmap.Create;
  Fbmp.Width := Width;
  Fbmp.Height := Height- 5;
  Gradfill (Fscolor, Fecolor, Fbmp);

  Text := 'GradGauge';

End;

Destructor Tgradgauge.Destroy;
Begin
  Inherited Destroy;
  Fbmp.Free;
End;

Procedure  Tgradgauge.SetRepresentation(Value: Boolean);
Begin
FRepresentationKind := Value;
Refreshcap := True;
Setprogress(Fprogress);
End;

{This procedure draws the gradient colors onto the bitmap
 Taken & slightly changed from WordCap by Warren F. Young - 10x ! }
Procedure Tgradgauge.Gradfill(Clr1, Clr2: Tcolor; Thebitmap: TBitmap);
Var
  Rgbfrom: Array[0..2] Of Byte;    { from RGB values                     }
  Rgbdiff: Array[0..2] Of Cardinal; { difference of from/to RGB values    }
  Colorband: TRect;                  { color band rectangular coordinates  }
  I: Cardinal;                { color band index                    }
  R: Byte;                   { a color band's R value              }
  G: Byte;                   { a color band's G value              }
  B: Byte;                   { a color band's B value              }
Begin
  { extract from RGB values}
  Rgbfrom[0] := GetRValue(Colortorgb(Clr1));
  Rgbfrom[1] := GetGValue(Colortorgb(Clr1));
  Rgbfrom[2] := GetBValue(Colortorgb(Clr1));
  { calculate difference of from and to RGB values}
  Rgbdiff[0] := GetRValue(Colortorgb(Clr2)) - Rgbfrom[0];
  Rgbdiff[1] := GetGValue(Colortorgb(Clr2)) - Rgbfrom[1];
  Rgbdiff[2] := GetBValue(Colortorgb(Clr2)) - Rgbfrom[2];
  { set pen sytle and mode}
  Thebitmap.Canvas.Pen.Style:= Pssolid;
  Thebitmap.Canvas.Pen.Mode:= Pmcopy;
  { set color band's left and right coordinates}
  Colorband.Left:= 0;
  Colorband.Right:= Thebitmap.Width;
  For I := 0 To $Ff Do
  Begin
    { calculate color band's top and bottom coordinates}
    Colorband.Top:= MulDiv (I, Thebitmap.Height, $100);
    Colorband.Bottom:= MulDiv (I + 1, Thebitmap.Height, $100);
    { calculate color band color}
    R := Rgbfrom[0] + MulDiv(I, Rgbdiff[0], $Ff);
    G := Rgbfrom[1] + MulDiv(I, Rgbdiff[1], $Ff);
    B := Rgbfrom[2] + MulDiv(I, Rgbdiff[2], $Ff);
    { select brush and paint color band}
    Thebitmap.Canvas.Brush.Color := RGB(R, G, B);
    Thebitmap.Canvas.Fillrect(Colorband);
  End;
End;

Procedure Tgradgauge.Paint;
Const Minus=3;
Begin
  If Not Visible Then Exit;

  Percent := Round(((Fprogress- Fmin) / (Fmax- Fmin)) * 100);
  Rp := (Percent * (Width-2)) Div 100;

  If Rp<>0 Then Begin
    Canvas.Copyrect (Rect(3, 3, Rp, Height-3),
      Fbmp.Canvas,
      Rect(0, 0, Fbmp.Width, Fbmp.Height));

//      Canvas.Pixels[4,0] := Fbgcolor;
//      Canvas.Pixels[Rp-1,0] := Fbgcolor;
  End;

  If Percent < 100 Then Begin
    Canvas.Brush.Color := Fbgcolor;
    Canvas.Brush.Style := Bssolid;
    Canvas.Pen.Style := Psclear;
    Canvas.Pen.Width := 1;
    Canvas.Rectangle (Rp+1, 2, Width, Height-1);
  End;

  If Fautocap Then Begin
    Canvas.Brush.Style := Bsclear;
    Canvas.Font := Font;
    Canvas.Font.Color :=cl3DDkShadow;
    R := Rect (2, 2, Width, Height);
    Drawtext(Canvas.Handle, Pchar(Text), Length(Text), R, Dt_Center Or Dt_Vcenter Or Dt_Singleline);
    R := Rect (0, 0, Width, Height);
    Canvas.Font := Font;
    Drawtext(Canvas.Handle, Pchar(Text), Length(Text), R, Dt_Center Or Dt_Vcenter Or Dt_Singleline);
  End;

  { }
  Canvas.Pen.Width := 1;
  Canvas.Pen.Color := clInactiveCaption;
  Canvas.Pen.Style := Pssolid;
  Canvas.Brush.Style := Bsclear;
{  Canvas.Rectangle(1, 1, Width, Height);}
  Canvas.MoveTo(Width-Minus,1);
  Canvas.LineTo(Minus,1);
  Canvas.LineTo(1,Minus);
  Canvas.LineTo(1,Height-Minus);
  Canvas.LineTo(Minus,Height-1);
  Canvas.Pen.Color := clBtnHighlight;
  Canvas.LineTo(Width-Minus,Height-1);
  Canvas.LineTo(Width-1,Height-Minus);
  Canvas.LineTo(Width-1,Minus);
  Canvas.LineTo(Width-Minus,1);
End;

Procedure Tgradgauge.Setbgcolor(Value:  Tcolor);
Begin
  If Value <> Fbgcolor Then
  Begin
    Fbgcolor := Value;
    Gradfill (Fscolor, Fecolor, Fbmp);
    Invalidate;
  End;
End;

Procedure Tgradgauge.Setcolor1(Value:  Tcolor);
Begin
  If Value <> Fscolor Then
  Begin
    Fscolor := Value;
    Gradfill (Fscolor, Fecolor, Fbmp);
    Invalidate;
  End;
End;

Procedure Tgradgauge.Setcolor2(Value:  Tcolor);
Begin
  If Value <> Fecolor Then
  Begin
    Fecolor := Value;
    Gradfill (Fscolor, Fecolor, Fbmp);
    Invalidate;
  End;
End;

Procedure Tgradgauge.Settext (Value: String);
 Begin
   If (Value <> Text) Then
   Begin
     Text := Value;
     Paint;
   End;
 End;

Procedure Tgradgauge.Setmin(Value:  Cardinal);
Begin
  If (Value <> Fmin) And (Value< Fmax) Then
  Begin
    Fmin := Value;
    If (Fprogress< Fmin) Then  Fprogress:= Fmin;
    Invalidate;
  End;
End;

Procedure Tgradgauge.Setmax(Value:  Cardinal);
Begin
  If (Value <> Fmax) And (Fmin< Value) Or (Value=0) Then
  Begin
    If Value=0 then Value:=1;

    Fmax := Value;
    If (Fprogress> Fmax) Then  begin
     Fprogress:= Fmax;
    end;
    
    Refreshcap := True;
    Setprogress (Fprogress);
    Invalidate;
  End;
End;

Procedure Tgradgauge.Setprogress(Value:  Cardinal);
var Temp: Int64;
Begin
  If (Value <> Fprogress) And (Value<= Fmax) And (Value >= Fmin) Or Refreshcap Then
  Begin
    Fprogress := Value;
    Temp :=Fprogress*100;
    Temp := Temp div Fmax;
    if (Temp>=99) And (FRepresentationKind) then
      Text := Inttostr (Fprogress) + '/' + Inttostr(Fmax)
    else
      Text := IntToStr (Temp)+'%';

    Refreshcap := false;
    Paint;
  End;
End;

Procedure Tgradgauge.Setautocap (Value:  Boolean);
Begin
  If (Value <> Fautocap)  Then
  Begin
    Fautocap := Value;
    Refreshcap := True;
    Setprogress (Fprogress);
    Invalidate;
  End;
End;

   procedure Tgradgauge.inc;
   begin
    setprogress (fprogress+1);
   end;

   procedure Tgradgauge.dec;
   begin
    setprogress (fprogress-1);
   end;

Procedure Register;
Begin
  Registercomponents('Visual', [TGradGauge] );
End;

End.
